import os
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.asymmetric import ec
from cryptography.hazmat.primitives.asymmetric.ec import ECDH
from cryptography.hazmat.primitives import serialization
from cryptography.hazmat.primitives.asymmetric import utils
from cryptography.exceptions import InvalidSignature


def generate_identity_keys():
    """
    Generate an ECDSA (P-256) identity key pair.
    Return (private_key, public_key_bytes).
    public_key_bytes is raw x||y each 32 bytes (uncompressed minus the 0x04 prefix).
    """
    private_key = ec.generate_private_key(ec.SECP256R1())
    public_numbers = private_key.public_key().public_numbers()

    x_bytes = public_numbers.x.to_bytes(32, byteorder='big')
    y_bytes = public_numbers.y.to_bytes(32, byteorder='big')
    pubkey_bytes = x_bytes + y_bytes  # 64 bytes

    return private_key, pubkey_bytes


def load_peer_identity_key(pubkey_bytes: bytes):
    """
    Given 64 bytes (x||y) for P-256, return a cryptography public key object.
    """
    if len(pubkey_bytes) != 64:
        raise ValueError("Peer identity pubkey must be exactly 64 bytes (x||y).")
    x_int = int.from_bytes(pubkey_bytes[:32], byteorder='big')
    y_int = int.from_bytes(pubkey_bytes[32:], byteorder='big')
    public_numbers = ec.EllipticCurvePublicNumbers(x_int, y_int, ec.SECP256R1())
    return public_numbers.public_key()


def sign_data(private_key, data: bytes) -> bytes:
    """
    Sign 'data' with ECDSA using P-256 private key.
    Returns DER-encoded signature (variable length, up to ~70-72 bytes).
    """
    signature = private_key.sign(data, ec.ECDSA(hashes.SHA256()))
    return signature


def verify_signature(public_key, signature: bytes, data: bytes) -> bool:
    """
    Verify DER-encoded ECDSA signature with the given public key.
    Return True if valid, False otherwise.
    """
    try:
        public_key.verify(signature, data, ec.ECDSA(hashes.SHA256()))
        return True
    except InvalidSignature:
        return False


def get_ephemeral_keypair():
    """
    Generate ephemeral ECDH keypair (P-256).
    Return (private_key, pubkey_bytes).
    """
    private_key = ec.generate_private_key(ec.SECP256R1())
    numbers = private_key.public_key().public_numbers()
    x_bytes = numbers.x.to_bytes(32, 'big')
    y_bytes = numbers.y.to_bytes(32, 'big')
    return private_key, x_bytes + y_bytes  # 64 bytes


def compute_ecdh_shared_key(private_key, peer_pubkey_bytes: bytes) -> bytes:
    """
    Given a local ECDH private_key and the peer's ephemeral pubkey (64 bytes),
    compute the shared secret.
    """
    x_int = int.from_bytes(peer_pubkey_bytes[:32], 'big')
    y_int = int.from_bytes(peer_pubkey_bytes[32:], 'big')
    peer_public_numbers = ec.EllipticCurvePublicNumbers(x_int, y_int, ec.SECP256R1())
    peer_public_key = peer_public_numbers.public_key()
    shared_key = private_key.exchange(ec.ECDH(), peer_public_key)
    return shared_key
